from PyQt5.QtWidgets import *
from pymirna.UI.mComboBox import *
from pymirna.UI.CutadaptUI import *
from pymirna.UI.ToolBlock import *
from pymirna.UI.FastxUI import *

FASTQTOFASTA = "fastq to fasta"
CUTADAPT = "Cutadapt"


class PreProcessingToolBlock(ToolBlock):
    def __init__(self, ppui, count):
        super(PreProcessingToolBlock, self).__init__(ppui, count)

        self._workflowStep = PREPROCESSING
        self._algorithmCombobox.addItem(CUTADAPT)
        self._algorithmCombobox.addItem(FASTQTOFASTA)

    def ChangeWidget(self):
        print("Widget Count : "+str(self._vboxlayout.count()))

        if(self._algorithmCombobox.getData() == "None"):
            widget = self._vboxlayout.takeAt(self._vboxlayout.count()-1).widget()
            widget.setVisible(False)
            widget = None
            self._vboxlayout.addWidget(QWidget())
            self._vboxlayout.update()
            self._curalgo = None
        elif(self._algorithmCombobox.getData() == CUTADAPT):
            widget = self._vboxlayout.takeAt(self._vboxlayout.count()-1).widget()
            widget.setVisible(False)
            ca = CutadaptUI(self, self.getID())
            self._vboxlayout.addWidget(ca)
            self._curalgo = ca
            self._vboxlayout.update()

        elif(self._algorithmCombobox.getData() == FASTQTOFASTA):
            widget = self._vboxlayout.takeAt(self._vboxlayout.count()-1).widget()
            widget.setVisible(False)

            ca = FastqtoFasta(self, self.getID())
            self._vboxlayout.addWidget(ca)
            self._curalgo = ca
            self._vboxlayout.update()




from PyQt5.QtWidgets import *
from pymirna.UI.mComboBox import *
from pymirna.UI.CutadaptUI import *
from pymirna.UI.ToolBlock import *
from pymirna.UI.miRandaUI import *
from pymirna.UI.FastxUI import *

MIRANDA = "miRanda"

class TargetPredictionToolBlock(ToolBlock):
    def __init__(self, ppui, count):
        super(TargetPredictionToolBlock, self).__init__(ppui, count)

        self._workflowStep = TARGETPREDICTION
        self._algorithmCombobox.addItem(MIRANDA)

    def ChangeWidget(self):
        print("Widget Count : "+str(self._vboxlayout.count()))

        if(self._algorithmCombobox.getData() == "None"):
            widget = self._vboxlayout.takeAt(self._vboxlayout.count()-1).widget()
            widget.setVisible(False)
            widget = None
            self._vboxlayout.addWidget(QWidget())
            self._vboxlayout.update()
            self._curalgo = None
        elif(self._algorithmCombobox.getData() == MIRANDA):
            widget = self._vboxlayout.takeAt(self._vboxlayout.count()-1).widget()
            widget.setVisible(False)

            #TODO CHANGE THIS
            ca = MirandaUI(self, self.getID())
            self._vboxlayout.addWidget(ca)
            self._curalgo = ca
            self._vboxlayout.update()






import sys

from pymirna.SeqIO.fastaReader import FastaReader

def main(dataset_filename):
    fasta = FastaReader(dataset_filename)
    print(fasta.getID())
    print(fasta.getSequence())
    while(not fasta.isEOF()):
        fasta.nextSequence()
        print(fasta.getID())
        print(fasta.getSequence())

if __name__=="__main__":
    main(sys.argv[1])  # read input filename from 1st command-line argument



from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from pymirna.Utility.FileReader import *

MODE_MULTI_SELECT = 0
MODE_SINGLE_SELECT = 1

class SelectInputDialog(QDialog):
    def __init__(self, p, mode):
        super(SelectInputDialog, self).__init__()
        self.__parent = p
        self.__initAttribute()
        self.__mode = mode
        self.__initScroll()
        self.__initUI()
        self.setMaximumSize(640, 480)
        self.setMinimumSize(320, 480)

    def __initAttribute(self):
        self.__scrollArea = None
        self.__scrollContent = None
        self.__scrollLayout = None

        self.__layout = None
        self.__okButton = None
        self.__cancelButton = None

        self.__filelist = []
        self.__selectFile = 0
        self.__mode = MODE_MULTI_SELECT

    def __initScroll(self):
        self.__scrollArea = QScrollArea(self)
        self.__scrollArea.setWidgetResizable(True)

        self.__scrollContent = QWidget(self.__scrollArea)

        self.__scrollArea.setWidget(self.__scrollContent)
        self.__scrollArea.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)

        self.__scrollLayout = QVBoxLayout(self.__scrollContent)
        self.__scrollLayout.setAlignment(Qt.AlignTop)
        self.__scrollLayout.setSpacing(5)

    def __initUI(self):
        self.__layout = QVBoxLayout(self)
        widget = QWidget()
        layout = QHBoxLayout(widget)
        widget.setLayout(layout)

        self.__okButton = QPushButton()
        self.__okButton.setText("OK")
        self.__okButton.clicked.connect(self.__okPressed)

        self.__cancelButton = QPushButton()
        self.__cancelButton.setText("Cancel")
        self.__cancelButton.clicked.connect(self.__cancelPressed)

        layout.addWidget(self.__okButton)
        layout.addWidget(self.__cancelButton)

        self.__layout.addWidget(self.__scrollArea)
        self.__layout.addWidget(widget)




    def addChoice(self, fileurl):
        newChoice = ChoiceBlock(fileurl, self)
        newChoice.setParent(self.__scrollContent)
        self.__scrollLayout.addWidget(newChoice)
        self.__filelist.append(newChoice)

    def getInputList(self):
        lis = []
        if(self.__mode == MODE_SINGLE_SELECT and self.__selectFile != None):
            return self.__selectFile.split('*')[0]

        for i in self.__filelist:
            if(i.isCheck()):
                lis.append(i.getFileURL().split('*')[0])
        return lis

    def __okPressed(self):
        self.__parent.getInputFromDialog(self)
        self.close()

    def __cancelPressed(self):
        self.__parent.getInputFromDialog(None)
        self.close()

    def elementChecked(self, cb):
        if(self.__mode == MODE_MULTI_SELECT):
            return
        if(self.__selectFile == 0):
            self.__selectFile = cb.getFileURL()
            return
        if(type(self.__selectFile) != ChoiceBlock):
            return
        self.__selectFile.setState(False)
        self.__selectFile = cb

class ChoiceBlock(QWidget):
    def __init__(self, fileurl, p):
        super(ChoiceBlock, self).__init__()

        self.__fileurl = fileurl
        self.__p = p
        self.__initUI()

    def __initUI(self):
        self.__layout = QHBoxLayout(self)
        self.__checkbox = QCheckBox()
        self.__label = QLabel(self.__fileurl)

        self.__layout.setAlignment(Qt.AlignLeft)
        self.__layout.setContentsMargins(0,0,0,0)
        self.__layout.setSpacing(5)
        self.__layout.addWidget(self.__checkbox)
        self.__layout.addWidget(self.__label)
        self.__checkbox.stateChanged.connect(self.__checked)

        self.setLayout(self.__layout)

    def isCheck(self):
        return self.__checkbox.isChecked()

    def getFileURL(self):
        return self.__fileurl

    def __checked(self):
        self.__p.elementChecked(self)

    def setState(self, state):
        self.__checkbox.setCheckState(state)


from PyQt5.QtWidgets import *


class mTextEdit(QTextEdit):
    def __init__(self):
        super(mTextEdit, self).__init__()

    def getData(self):
        return QTextEdit.toPlainText()

    def setData(self, data):
        if(type(data) == str):
            self.setText(data)


__author__ = 'haru'

from PyQt5 import QtWidgets as w
from PyQt5 import QtGui as g
from PyQt5 import QtCore as c

from pymirna.UI.PreProcessUI import *
from pymirna.UI.TargetPrediction import *
from pymirna.UI.fileViewer import *
from pymirna.Utility.ProjectUtility import *
from pymirna.UI.mTreeWidgetItem import *
from pymirna.UI.Discovery import *
from pymirna.UI.GlobalVariable import *

OUTPUT_TAB_ID = 3
DEBUGGING = True

class MainPage(w.QMainWindow):
    def __init__(self, title = "miRWB", width = 1920, height = 1080):
        super(MainPage, self).__init__()
        self.setWindowTitle(title)
        self.setMaximumSize(width, height)

        geo = QDesktopWidget().screenGeometry()
        self.setGeometry(0,0,geo.width() * 0.7, geo.height()*(0.7))
        self.__initAttribute__()
        self.__initUI__()

        self.show()

    def __initAttribute__(self):
        self.__menubar                = None
        self.__fileMenu             = None
        self.__toolMenu             = None
        self.__exitAction           = None
        self.__removeFileAction     = None
        self.__algoCompareAction    = None
        self.__createProjectAction  = None
        self.__addFileAction        = None
        self.__openProjectAction    = None
        self.__closeProjectAction   = None
        self.__leftContainer        = None
        self.__leftScrollArea       = None
        self.__leftVContainer       = None
        self.__leftVLayout          = None
        self.__outputContainer      = None
        self.__centerVContainer     = None
        self.__rightContainer       = None
        self.__rightVContainer      = None
        self.__preTab               = None
        self.__discoveryTab         = None
        self.__predictTab           = None
        self.__outputTab            = None
        self.__outputLayout         = None
        self.__projectUtility       = None
        self.__projectmanager       = None
        self.__isOpenProject        = False
        self.__initPreID            = 0
        self.__initPredictID        = 0
        self.__initDiscoveryID      = 0


    def __initUI__(self):

        self.__initAction__()
        self.__initMenuBar__()
        self.__initContainer__()
        self.__initLayout__()
        self.__initList__()
        self.__projectUtility = ProjectManager()
        self.__preTab.setProjectManager(self.__projectUtility)
        self.__preTab.setInitialID(self.__initPreID)
        self.__discoveryTab.setProjectManager(self.__projectUtility)
        self.__discoveryTab.setInitialID(self.__initDiscoveryID)
        self.__predictTab.setProjectManager(self.__projectUtility)
        self.__predictTab.setInitialID(self.__initPredictID)



    def __initMenuBar__(self):
        # self.statusBar()

        # ADD ELEMENT TO MENU BAR
        self.__menubar = w.QMenuBar()

        self.__fileMenu = self.__menubar.addMenu('&File')
        self.__toolMenu = self.__menubar.addMenu('&Tool')

        # ADD ACTION TO MENU BAR
        self.__fileMenu.addAction(self.__createProjectAction)
        self.__fileMenu.addAction(self.__openProjectAction)
        self.__fileMenu.addAction(self.__addFileAction)
        self.__fileMenu.addAction(self.__removeFileAction)
        self.__fileMenu.addAction(self.__closeProjectAction)
        self.__fileMenu.addAction(self.__exitAction)
        self.__toolMenu.addAction(self.__algoCompareAction)

    def __initAction__(self):
        self.__exitAction = w.QAction('Exit', self)
        self.__exitAction.triggered.connect(self.close)

        self.__algoCompareAction = w.QAction('Performance Comparison', self)

        self.__createProjectAction = w.QAction('Create Project', self)
        self.__createProjectAction.triggered.connect(self.__createProject)

        self.__openProjectAction = w.QAction('Open Project', self)
        self.__openProjectAction.triggered.connect(self.__openProject)

        self.__closeProjectAction = w.QAction('Close Project', self)
        self.__closeProjectAction.triggered.connect(self.__closeProject)
        self.__closeProjectAction.setDisabled(True)

        self.__addFileAction = w.QAction('Add File', self)
        self.__addFileAction.triggered.connect(self.__addFileTriggered)
        self.__addFileAction.setDisabled(True)

        self.__removeFileAction = w.QAction('Remove File', self)
        self.__removeFileAction.triggered.connect(self.__removeFileTriggered)
        self.__removeFileAction.setDisabled(True)


    def __initContainer__(self):

        # INITIALIZE LEFT CONTAINER
        self.__leftContainer = w.QTabWidget()
        self.__leftContainer.setMaximumWidth(self.width() * 0.22)

        # INITIALIZE RIGHT VERTICLE CONTAINER
        self.__rightVContainer = w.QVBoxLayout(self.__rightContainer)

        # CREATE WORKFLOW TAB
        self.__preTab = PreProcessUI(self)
        self.__discoveryTab = Discovery(self)
        self.__predictTab = TargetPrediction(self)
        self.__outputTab = w.QWidget()
        self.__outputTab.setStyleSheet("background-color: white;")

        # INITIALIZE LEFT VERTICLE CONTAINER
        self.__leftVContainer = w.QWidget()
        self.__leftScrollArea = w.QScrollArea()
        self.__leftScrollArea.setWidget(self.__leftVContainer)
        self.__leftContainer.addTab(self.__leftVContainer, "Project Manager")

        # INITIALIZE RIGHT CONTAINER
        self.__rightContainer = w.QTabWidget()
        self.__rightContainer.setElideMode(3)
        self.__rightContainer.setUsesScrollButtons(True)


        # INITIALIZE CENTRAL CONTAINER
        self.__centralContainer = w.QWidget()
        self.setCentralWidget(self.__centralContainer)

        # ADD WORKFLOW TAB TO RIGHT CONTAINER
        self.__rightContainer.addTab(self.__preTab, "Pre-Processing")
        self.__rightContainer.addTab(self.__discoveryTab, "miRNA Discovery")
        self.__rightContainer.addTab(self.__predictTab, "Target Prediction")
        self.__rightContainer.addTab(self.__outputTab, "Output")

        # INITIALIZE CENTER CONTAINER

        self.__outputContainer = w.QTabWidget()
        self.__outputContainer.setStyleSheet("background-color: white;")
        self.__outputLayout = QVBoxLayout(self.__outputTab)
        self.__outputLayout.setAlignment(Qt.AlignTop)
        self.__outputLayout.setContentsMargins(0,0,0,0)
        self.__outputLayout.setSpacing(0)
        self.__outputContainer.setTabsClosable(True)
        self.__outputContainer.tabCloseRequested.connect(self.__closeCenterTab)
        #
        # # INITIALIZE CENTER VERTICLE CONTAINER
        # self.__centerVContainer = w.QVBoxLayout()
        # self.__centerVContainer.addChildWidget(self.__outputContainer)



    def __initLayout__(self):
        layout = w.QHBoxLayout(self.__centralContainer)
        layout.setContentsMargins(0,0,0,0)
        layout.setSpacing(1)
        layout.addWidget(self.__leftContainer)
        layout.addWidget(self.__rightContainer)
        # self.__outputTab.setLayout(self.__centerVContainer)
        self.__outputLayout.addWidget(self.__outputContainer)


    #TODO Working on here
    def __initList__(self):

        self.__leftVLayout = QVBoxLayout(self.__leftVContainer)
        self.__projectmanager = w.QTreeWidget()
        self.__projectmanager.setFocusPolicy(0)
        self.__projectmanager.header().close()
        self.__projectmanager.setTextElideMode(c.Qt.ElideNone)
        self.__projectmanager.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        self.__projectmanager.header().setSectionResizeMode(w.QHeaderView.ResizeToContents)
        self.__projectmanager.header().setStretchLastSection(False)
        self.__projectmanager.setSortingEnabled(True)
        self.__projectmanager.itemSelectionChanged.connect(self.__selectFile)
        self.__leftVLayout.addWidget(self.__projectmanager)
        self.__leftVLayout.setContentsMargins(0,0,0,0)
        self.__leftVLayout.setAlignment(Qt.AlignTop)

        self.__leftVContainer.setLayout(self.__leftVLayout)

        # # #Test Add Item
        # par = w.QTreeWidgetItem(self.__projectmanager, ["Folder1"])
        # self.__projectmanager.addTopLevelItem(par)
        # chi = w.QTreeWidgetItem(par, ["File1.fasdfasdfasdfasta".format(1)])
        # self.__projectmanager.addTopLevelItem(chi)
        # par.addChild(chi)
        # test = w.QTreeWidgetItem(chi, ["File2asdfasdfasd.fasta".format(1)])
        # self.__projectmanager.addTopLevelItem(test)
        # par.addChild(test)

        self.__projectmanager.doubleClicked.connect(self.__DoubleClickTreeItem)

    #TODO THIS ONE IS USABLE
    def __DoubleClickTreeItem(self, index):
        if(DEBUGGING):
            print("Type of Index"+str(type(index)))
            print("Double Click: " + str(self.__outputContainer.count()), end="")
            print(index.data())
        if(index.data() == self.__projectUtility.getProjectName()): #CHECK IF USER DOUBLE CLICK THE PROJECT NAME
            return

        item = self.__projectmanager.findItems(index.data(), c.Qt.MatchExactly | c.Qt.MatchRecursive)
        index = item[0]

        if(isFolder(index.getFilePath())):
            return False

        #Check if the file is already open
        if(self.__outputContainer.count() is 0):
            self.__rightContainer.setCurrentIndex(OUTPUT_TAB_ID)
            self.__outputContainer.addTab(FileViewer(index.getFilePath(), self.__outputContainer), index.getName())
            return

        for i in range (0,self.__outputContainer.count() ):
            if(self.__outputContainer.widget(i).getFile() == index.getFilePath()):
                self.__rightContainer.setCurrentIndex(OUTPUT_TAB_ID)
                self.__outputContainer.setCurrentIndex(i)
                return

        self.__outputContainer.addTab(FileViewer(index.getFilePath(), self.__outputContainer), index.getName())
        self.__rightContainer.setCurrentIndex(OUTPUT_TAB_ID)
        self.__outputContainer.setCurrentIndex(self.__outputContainer.count()-1)

    #TODO THIS ONE DOING FINE
    def __closeCenterTab(self, index):
        print("Index : " + str(index))
        if( self.__outputContainer.widget(index).isUnsaved() ):
            print("All unsaved data will be loss. ")
        else:
            print("You are fine")
        self.__outputContainer.removeTab(index)

    def __createProject(self):
        path = w.QFileDialog.getSaveFileName(self, "Select Directory")
        if(len(path[0]) <= 0):
            return
        name = getFileName(path[0])
        self.__projectUtility.createProject(path[0][0:len(path[0])-len(name)], name)
        print(self.__projectUtility.getProjectName())
        print(self.__projectUtility.getDirectory())

        self.__initProjectManager()
        self.__toggleWidget()

    def __openProject(self):
        path = str(w.QFileDialog.getOpenFileUrl(self, "Select Project", "/", self.tr("Project file (*.pmi)")))

        lis = path.split("'")
        path = lis[1][7:len(lis[1]):]
        if(len(path) <= 0):
            if(DEBUGGING):
                print("No file selected")
            return

        print("Path : "+path)
        self.__projectUtility.openProject(path)

        if(DEBUGGING):
            print("Open project: project name: "+self.__projectUtility.getProjectName())
        self.__preTab.setInitialID(self.__projectUtility.getInitialID()[0])
        self.__discoveryTab.setInitialID(self.__projectUtility.getInitialID()[1])
        self.__predictTab.setInitialID(self.__projectUtility.getInitialID()[2])
        self.__initProjectManager()
        self.__toggleWidget()

        if(DEBUGGING):
            print("Open project: project directory: "+self.__projectUtility.getDirectory())

    def __closeProject(self):
        print("Close Project")
        self.__initAttribute__()
        self.__initUI__()
        self.__toggleWidget()
        self.__isOpenProject = False

    def __addFileTriggered(self):
        path = str(w.QFileDialog.getOpenFileName(self, "Select File"))

        if(len(path) == 0):
            return

        lis = path.split("'")
        ismove = QMessageBox.question(self, "Message", "Do you want to move selected file to filelist folder in project directory ?", QMessageBox.Yes , QMessageBox.No)
        if(ismove == QMessageBox.Yes):
            cmd = "cp "+lis[1]+" "+self.__projectUtility.getDirectory()+"file_list/"+getFileName(lis[1])
            os.system(cmd)
            lis = self.__projectUtility.getDirectory()+"file_list/"+getFileName(lis[1])

        self.__projectUtility.addFile(lis)
        self.__projectUtility.saveProject()
        if(DEBUGGING):
            print(lis)
            print("addFileTriggered : File List : ", end = '')
            print(self.__projectUtility.getFileList())
        previousItem = self.__projectmanager.findItems(self.__projectUtility.getProjectName(), c.Qt.MatchRecursive | c.Qt.MatchExactly)[0]
        dirlist = getDirectoryList(lis)
        startindex = 0
        for i in range(0, len(dirlist)):
            if(dirlist[i] == self.__projectUtility.getProjectName()):
                startindex = i+1
                break
            elif(dirlist[i] == "file_list"):
                startindex = i
                break
        for i in range(startindex, len(dirlist)):
            item = self.__projectmanager.findItems(dirlist[i], c.Qt.MatchRecursive | c.Qt.MatchExactly)
            if(len(item) == 0):
                i = mTreeWidgetItem(previousItem, [dirlist[i]], '', FOLDER)
                previousItem = i
            else:
                previousItem = item[0]


        i = mTreeWidgetItem(previousItem, [getFileName(lis)], lis[0:len(lis) - len(getFileName(lis)):], FILE )
        self.__projectmanager.addTopLevelItem(i)

    def __initProjectManager(self):
        self.__projectmanager.clear()

        proj = mTreeWidgetItem(self.__projectmanager, [self.__projectUtility.getProjectName()], '', PROJECT)
        self.__projectmanager.addTopLevelItem(proj)
        a = self.__projectUtility.getFileList()
        for i in getFileList(self.__projectUtility.getDirectory()):
            if(i not in a):
                a.append(i)

        for i in a:
            if(len(self.__projectmanager.findItems(getFileName(i), c.Qt.MatchRecursive | c.Qt.MatchExactly) ) > 0):
                continue

            previousItem = self.__projectmanager.findItems(self.__projectUtility.getProjectName(), c.Qt.MatchRecursive | c.Qt.MatchExactly)[0]
            dirlist = getDirectoryList(i)
            startindex = 0
            for j in range(0, len(dirlist)):
                if(dirlist[j] == self.__projectUtility.getProjectName()):
                    startindex = j+1
                    break
                elif(dirlist[j] == "file_list"):
                    startindex = j
                    break
                elif(j == len(dirlist)-1):
                    startindex = len(dirlist)
            for j in range(startindex, len(dirlist)):
                item = self.__projectmanager.findItems(dirlist[j], c.Qt.MatchRecursive | c.Qt.MatchExactly)
                if(len(item) == 0):
                    item = mTreeWidgetItem(previousItem, [dirlist[j]], '', FOLDER)
                    previousItem = item
                else:
                    previousItem = item[0]

            inItem = mTreeWidgetItem(previousItem, [getFileName(i)], i[0:len(i)-len(getFileName(i))], FILE)

    def __toggleWidget(self):
        self.__addFileAction.setDisabled(self.__isOpenProject)
        self.__closeProjectAction.setDisabled(self.__isOpenProject)
        self.__preTab.setWidgetEnable(self.__isOpenProject)

    def __selectFile(self):
        index = self.__projectmanager.selectedIndexes()
        if(len(index) > 0):
            index = index[0]
            item = self.__projectmanager.itemFromIndex(index)
            if(item.childCount() == 0):
                self.__removeFileAction.setEnabled(True)
            elif(item.childCount() != 0):
                self.__removeFileAction.setEnabled(False)


    def __removeFileTriggered(self):
        index = self.__projectmanager.selectedIndexes()[0]
        if(DEBUGGING):
            print("Remove File : "+index.data())
        item = self.__projectmanager.itemFromIndex(index)
        item.parent().removeChild(item)

    def checkBoxChanged(self, param):
        self.__preTab.setAutoWorkflow(param)
        self.__predictTab.setAutoWorkflow(param)
        self.__discoveryTab.setAutoWorkflow(param)

    def updateProject(self):
        self.__initProjectManager()
        self.__projectUtility.setInitialID([self.__preTab.getInitialID(), self.__discoveryTab.getInitialID(), self.__predictTab.getInitialID()])
        self.__projectUtility.saveProject()

    def runWorkflow(self):
        if(DEBUGGING):
            print("runWorkflow")
        toolqueue = self.__preTab.getToolQueue()
        toolqueue += self.__discoveryTab.getToolQueue()
        toolqueue += self.__predictTab.getToolQueue()
        for i in toolqueue:
            i.runCommand()
        #TODO INIT PROGRESS DIALOG

        #TODO COLLECT ALL COMMAND

        #TODO SEND COMMANDS TO DIALOG

        #LET DIALOG DO THE JOB

    def requestExpectedOutput(self, wf, id, q):
        toolQueue = q
        lis = []
        if(DEBUGGING):
            print("toolQueue : ", end='')
            print(toolQueue)

        if(toolQueue == None):
            return
        for i in toolQueue:
            if(i.getWorkflow() > wf):
                break
            if(i.getID() >= id):
                continue
            lis += i.getExpectedOutput()
        return lis



import PyQt5.QtWidgets as w


class mComboBox(w.QComboBox):

    def __init__(self):
        super(mComboBox, self).__init__()

    def getData(self):
        return self.currentText()

    def setData(self, value):
        for i in eval(self.count()):
            if(value == self.itemData(i)):
                self.setCurrentIndex(i)
                return
        print(value+" not found in the combobox")




from PyQt5 import QtWidgets as w


FILE = 1
FOLDER = 2
PROJECT = 3

class mTreeWidgetItem(w.QTreeWidgetItem):
    def __init__(self, p, lis, fpath, t):
        super(mTreeWidgetItem, self).__init__(p)
        self.__fpath = fpath
        self.__name = lis[0]
        self.setText(0, self.__name)

        if(t < 1 or t > 3):
            t = 1
        self.__type = t

    def getFilePath(self):
        return self.__fpath+self.__name

    def setType(self, t):
        if(t < 1 or t > 3):
            t = 1
            self.__type = t

    def getType(self):
        return self.__type

    def getName(self):
        return self.__name


import PyQt5.QtWidgets as w


class SettingBlock (w.QHBoxLayout):



    def __init__(self, labelName, relement = None):
        super(SettingBlock, self).__init__()
        self.__initAttribute__(labelName)
        if(relement is not None or relement is not w.QWidget ):
            self.__rightelement = relement
            self.addWidget(self.__rightelement)


    def __initAttribute__(self, labelName):
        self.__label = w.QLabel(labelName)
        self.addWidget(self.__label)
        self.__rightelement = None

    def addWidgetToLayout(self, widget):
        if(self.__rightelement == None):
            self.__rightelement = widget
            self.addWidget(self.__rightelement)

        elif(self.__rightelement is not None):
            temp = self.__rightelement
            self.__rightelement = []
            self.__rightelement.append(temp)
            self.addWidget(widget)
            self.__rightelement.append(widget)

        elif(type(self.__rightelement) == list):
            self.addWidget(widget)
            self.__rightelement.append(widget)

    def checkIdentity(self, id):
        if(id == self.__label.text()):
            return True
        return False

    def getData(self):
        if(type(self.__rightelement) == list):
            lis = []
            for _ in self.__rightelement:
                lis.append(self.__rightelement.getData())
            return lis
        return self.__rightelement.getData()

    def setData(self, value):
        if(type(value) == list):
            for i in value:
                self.__rightelement.setData(i)
        else:
            self.__rightelement.setData(value)

    def changeIdentity(self, name):
        if(len(name) == 0):
            return
        self.__label.setText(name)


from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from pymirna.UI.SettingBlock import *
from pymirna.UI.mComboBox import *
from pymirna.UI.mTextEdit import *
from pymirna.UI.SelectInputDialog import *
from pymirna.UI.GlobalVariable import *
import os
import subprocess as sub


ERROR_RATE_DEFAULT = 0.1
REMOVE_N_DEFAULT = 1

class CutadaptUI(QWidget):
    def __init__(self, p, id):
        super(CutadaptUI, self).__init__()
        self.__parent = p
        self.__initAttribute__()
        self.__initUI()
        self.__ID = id
        self.setVisible(True)


    def __initAttribute__(self):
        self.__vboxlayout       = None
        self.__primeCombobox    = None
        self.__adapterTextEdit  = None
        self.__inputButton = None
        self.__errorRateTextEdit = None
        self.__removenTextEdit = None
        self.__expectedoutput = None
        self.__inputFile = []
        self.__inputSetting = None

    def __initUI(self):
        self.__initFrame()
        self.__initLayout()

    def __initFrame(self):
        self.__inputButton = QPushButton("Select Input File")
        self.__inputButton.setMaximumHeight(30)
        self.__inputButton.clicked.connect(self.__inputButtonPressed)

        self.__primeCombobox = mComboBox()
        self.__primeCombobox.addItem("3'")
        self.__primeCombobox.addItem("5'")

        self.__adapterTextEdit = mTextEdit()
        self.__adapterTextEdit.setMaximumHeight(30)

        self.__errorRateTextEdit = mTextEdit()
        self.__errorRateTextEdit.setMaximumHeight(30)

        self.__removenTextEdit = mTextEdit()
        self.__removenTextEdit.setMaximumHeight(30)

    def __initLayout(self):
        self.__vboxlayout = QVBoxLayout(self)
        self.__vboxlayout.setAlignment(Qt.AlignTop)
        self.__inputSetting = SettingBlock("Input File*", self.__inputButton)
        self.__vboxlayout.addLayout(self.__inputSetting)
        self.__vboxlayout.addLayout(SettingBlock("Type of Adapter", self.__primeCombobox))
        self.__vboxlayout.addLayout(SettingBlock("Adapter Sequence*", self.__adapterTextEdit))
        self.__vboxlayout.addLayout(SettingBlock("Maxximum error Rate (Default = 0.1)", self.__errorRateTextEdit))
        self.__vboxlayout.addLayout(SettingBlock("Remove up to n adapters from each read (Default = 1)", self.__removenTextEdit))

        self.setLayout(self.__vboxlayout)

    def runCommand(self):
        lis = []
        if(len(self.__inputFile) == 0):
            QMessageBox.question(QWidget(), "Alert", "No input file selected for Cutadapt ID: " + str(self.__ID+1), QMessageBox.Ok)
            return 0
        elif(len(self.__adapterTextEdit.toPlainText()) == 0):
            QMessageBox.question(self, "Alert", "No adapter sequence enter for Cutadapt ID: " + str(self.__ID+1), QMessageBox.Ok)
            return 0
        os.system("mkdir -p "+self.__parent.requestProjectDirectory()+"Result/PreProcessing/cutadapt/")
        for i in self.__inputFile:
            ret = "python3 algorithm/preprocess/cutadapt/cutadapt "
            if(self.__primeCombobox.getData() == "3'"):
                ret += "-a "
            else:
                ret += "-g "
            if(len(self.__adapterTextEdit.toPlainText()) > 0):
                ret += self.__adapterTextEdit.toPlainText()+" "

            if(len(self.__errorRateTextEdit.toPlainText()) > 0):
                ret += "-e "+self.__errorRateTextEdit.toPlainText()+" "

            if(len(self.__removenTextEdit.toPlainText()) > 0):
                ret += "-n "+self.__removenTextEdit.toPlainText()+" "

            ret += "-o "+self.__parent.requestProjectDirectory()+"Result/PreProcessing/cutadapt/"+getFilenameWithoutExtension(i)+"_Cutadapt_"+str(self.__ID)+"."+getFileExtension(i)+" "

            ret += " "+i
        if(DEBUGGING):
            print(ret)

        log = os.popen(ret)
        if(len(log) > 0):
            ol = open(self.__parent.requestProjectDirectory()+"Result/PreProcessing/cutadapt/"+getFilenameWithoutExtension(i)+"_Cutadapt_"+str(self.__ID)+".log", 'w')
            ol.writelines(log)
            ol.close()


    def getExpectedOutput(self):
        lis = []
        print("INPUT FILE : ", end='')
        print(self.__inputFile)
        if(type(self.__inputFile) is not type([])):
            return []
        for file in self.__inputFile:
            print(file)
            lis.append(self.__parent.requestProjectDirectory()+"Result/PreProcessing/"+getFilenameWithoutExtension(file)+"_Cutadapt_"+str(self.__ID)+getFileExtension(file))
        return lis

    def __inputButtonPressed(self):
        print("Select input button pressed")
        selectinput = SelectInputDialog(self, MODE_MULTI_SELECT)
        lis = self.__parent.requestFileList()
        if(type(lis) == type([])):
            filelist = lis
        lis = self.__parent.requestExpectedOutput()
        if(type(lis) == type([])):
            filelist += lis
        for i in filelist:
            if(i[len(i)-1] == '*'):
                if(getFileExtension(i[0:len(i)-1])):
                    selectinput.addChoice(i)
                    continue
            if(getFileExtension(i) == "fasta" or getFileExtension(i[0:len(i)-1]) == "fastq"):
                selectinput.addChoice(i)
        selectinput.exec_()
        # selectinput.show()

    def getInputFromDialog(self, inputDialog):
        if(inputDialog == None):
            return

        self.__inputFile = inputDialog.getInputList()
        if(len(self.__inputFile) == 0):
            self.__inputSetting.changeIdentity("Input File*")
        else:
            self.__inputSetting.changeIdentity("Input File")



from pymirna.UI.TargetPredictionToolBlock import *
from pymirna.UI.mComboBox import *
from pymirna.UI.mTextEdit import *
from pymirna.UI.mTreeWidgetItem import *
from pymirna.UI.GlobalVariable import *

ERROR_RATE_DEFAULT = 0.1
REMOVE_N_DEFAULT = 1

class MirandaUI(QWidget):
    def __init__(self, p, id):
        super(MirandaUI, self).__init__()
        self.__parent = p
        self.__initAttribute__()
        self.__initUI()
        self.__ID = id
        self.setVisible(True)


    def __initAttribute__(self):
        self.__vboxlayout       = None

        self.__inputQuery = None
        self.__inputSequences = []
        self.__inputSequencesSetting = None
        self.__inputQueryButton = None
        self.__inputQuerySetting = None
        self.__inputSequencesButton = None
        self.__scoreTextedit = None
        self.__energyTextedit = None
        self.__scalingTextedit = None
        self.__gap_openTextedit = None
        self.__gap_extendTextedit = None
        self.__inputMode = None

    def __initUI(self):
        self.__initFrame()
        self.__initLayout()



    def __initFrame(self):
        self.__inputQueryButton = QPushButton()
        self.__inputQueryButton.setText("Select input query file")
        self.__inputQueryButton.setMaximumHeight(30)
        self.__inputSequencesButton = QPushButton()
        self.__inputSequencesButton.setText("Select input sequences files")
        self.__inputSequencesButton.setMaximumHeight(30)
        self.__scoreTextedit = mTextEdit()
        self.__scoreTextedit.setMaximumHeight(30)
        self.__energyTextedit = mTextEdit()
        self.__energyTextedit.setMaximumHeight(30)
        self.__scalingTextedit = mTextEdit()
        self.__scalingTextedit.setMaximumHeight(30)
        self.__gap_extendTextedit = mTextEdit()
        self.__gap_extendTextedit.setMaximumHeight(30)
        self.__gap_openTextedit = mTextEdit()
        self.__gap_openTextedit.setMaximumHeight(30)

        self.__inputQueryButton.clicked.connect(self.__inputQueryPressed)
        self.__inputSequencesButton.clicked.connect(self.__inputSequencesPressed)

    def __initLayout(self):
        self.__vboxlayout = QVBoxLayout(self)
        self.__vboxlayout.setAlignment(Qt.AlignTop)

        self.__inputQuerySetting = SettingBlock("Input query*", self.__inputQueryButton)
        self.__inputSequencesSetting = SettingBlock("Input sequence*", self.__inputSequencesButton)

        self.__vboxlayout.addLayout(self.__inputQuerySetting)
        self.__vboxlayout.addLayout(self.__inputSequencesSetting)
        self.__vboxlayout.addLayout(SettingBlock("Score threshold (Default = 140.0)", self.__scoreTextedit))
        self.__vboxlayout.addLayout(SettingBlock("Energy threshold in kcal/mol (Default = 1.0)", self.__energyTextedit))
        self.__vboxlayout.addLayout(SettingBlock("Scaling parameter (Default = 4.0)", self.__scalingTextedit))
        self.__vboxlayout.addLayout(SettingBlock("Gap-open penalty (Default = -4.0)", self.__gap_openTextedit))
        self.__vboxlayout.addLayout(SettingBlock("Gap-extend penalty (Default = -9.0", self.__gap_extendTextedit))

        self.setLayout(self.__vboxlayout)

    def runCommand(self):
        lis = []
        os.system("mkdir -p "+self.__parent.requestProjectDirectory()+"Result/TargetPrediction/miRanda/")

        if(len(self.__inputSequences) == 0):
            QMessageBox.question(self, "Alert", "No input sequence for miRanda"+str(self.__ID+1), QMessageBox.Ok)
            return
        if(len(self.__inputQuery) == 0):
            QMessageBox.question(self, "Alert", "No input Query for miRanda"+str(self.__ID+1), QMessageBox.Ok)
            return

        for i in self.__inputSequences:
            ret = "algorithm/targetprediction/miranda/miranda "

            ret += self.__inputQuery + " "

            ret += i + " "

            if(len(self.__scoreTextedit.toPlainText()) > 0):
                ret += "-sc "+self.__scoreTextedit.toPlainText()+" "

            if(len(self.__energyTextedit.toPlainText()) > 0):
                ret += "-en "+self.__energyTextedit.toPlainText()+" "

            if(len(self.__scalingTextedit.toPlainText()) > 0):
                ret += "-scale "+self.__scalingTextedit.toPlainText()+" "

            if(len(self.__gap_extendTextedit.toPlainText()) > 0):
                ret += "-ge "+self.__gap_extendTextedit.toPlainText()+" "
            if(len(self.__gap_openTextedit.toPlainText()) > 0):
                ret += "-go "+self.__gap_openTextedit.toPlainText()+" "

            ret += "-out "+self.__parent.requestProjectDirectory()+"Result/TargetPrediction/miranda/"+getFilenameWithoutExtension(i)+"_miRanda_"+str(self.__ID)+".txt"

        if(DEBUGGING):
            print(ret)

        log = os.popen(ret)
        if(len(log) > 0):
            ol = open(self.__parent.requestProjectDirectory()+"Result/TargetPrediction/miranda/"+getFilenameWithoutExtension(i)+"_miRanda_"+str(self.__ID)+".log", 'w')
            ol.writelines(log)
            ol.close()

    def getExpectedOutput(self):
        return []

    def __inputSequencesPressed(self):
        print("Select input sequences button pressed")
        self.__inputMode = "seq"
        selectinput = SelectInputDialog(self, MODE_MULTI_SELECT)
        lis = self.__parent.requestFileList()
        if(type(lis) == type([])):
            filelist = lis
        lis = self.__parent.requestExpectedOutput()
        if(type(lis) == type([])):
            filelist += lis
        for i in filelist:
            if(i[len(i)-1] == '*'):
                if(getFileExtension(i[0:len(i)-1])):
                    selectinput.addChoice(i)
                    continue
            if(getFileExtension(i) == "fasta"):
                selectinput.addChoice(i)
        selectinput.exec_()
        # selectinput.show()

    def __inputQueryPressed(self):
        print("Select input query button pressed")
        self.__inputMode = "que"
        selectinput = SelectInputDialog(self, MODE_SINGLE_SELECT)
        lis = self.__parent.requestFileList()
        if(type(lis) == type([])):
            filelist = lis
        lis = self.__parent.requestExpectedOutput()
        if(type(lis) == type([])):
            filelist += lis
        for i in filelist:
            if(i[len(i)-1] == '*'):
                if(getFileExtension(i[0:len(i)-1])):
                    selectinput.addChoice(i)
                    continue
            if(getFileExtension(i) == "fasta"):
                selectinput.addChoice(i)
        selectinput.exec_()
        # selectinput.show()

    def getInputFromDialog(self, inputDialog):
        if(inputDialog == None):
            return
        if(self.__inputMode == "seq"):
            self.__inputSequences = inputDialog.getInputList()
            if(len(self.__inputSequences) == 0):
                self.__inputSequencesSetting.changeIdentity("Input sequence*")
                return
            else:
                self.__inputSequencesSetting.changeIdentity("Input sequence")
                return


        elif(self.__inputMode == 'que'):
            self.__inputQuery = inputDialog.getInputList()
            if(len(self.__inputQuery) == 0):
                self.__inputQuerySetting.changeIdentity("Input query*")
                return
            else:
                self.__inputQuerySetting.changeIdentity("Input query")
                return



__author__ = 'SupanatK'

import os
import sys
import xml.etree.cElementTree
import subprocess


from pymirna.UI.MainPage import *
from pymirna.UI.PreProcessUI import *
from pymirna.UI.PreProcessUI import *
from pymirna.Utility import *
from pymirna.UI.SelectInputDialog import *

if(__name__ == '__main__'):

    # TODO FINISH THIS IN TIME DUDE!!
    app = w.QApplication([])
    app.setApplicationName("miRWB")
    mainpage = MainPage()
    mainpage.show()
    sys.exit(app.exec_())


__author__ = 'haru'

import PyQt5.QtWidgets as w
import PyQt5.QtCore as c


class FloatingWidget(w.QWidget):

    def __init__(self, title = "Floating Windows", width = 640, height = 480):
        super(FloatingWidget, self).__init__()
        self.setWindowTitle(title)
        self.setFixedSize(width, height)


    def __initAttribute__(self):
        Attributegoeshere = "Test"



import PyQt5.QtWidgets as w
from PyQt5.QtCore import *
import pymirna.UI.SettingBlock as s
from pymirna.UI.mComboBox import *
from pymirna.UI.GlobalVariable import *

from pymirna.UI.PreProcessingToolBlock import *
import os
import sys

class WorkflowUI(w.QWidget):

    def __init__(self, p):
        super(WorkflowUI, self).__init__()
        self.__mainpage = p
        self.__initAttribute()
        self.__initScrolContent()
        self.__initCommencingBlock()
        self.__initLayout__()
        self.setVisible(True)
        self.setContentsMargins(0,0,0,0)
        self.count = 0
        self.__saveProfileButton.setVisible(False)

    def __initAttribute(self):
        self.__commencingBlock = None
        self.__autoCheckBox = None
        self.__startButton = None
        self.__saveProfileButton = None
        self.__addAlgoButton = None
        self._vboxlayout = None
        self.__vscrollArea = None
        self._vscrollContent = None
        self._vscrollLayout = None
        self._toolQueue = []
        self._toolcount = 0
        self.__projectmanager = None
        self._initID = 0

    def __initLayout__(self):
        self._vboxlayout = w.QVBoxLayout(self)
        self._vscrollLayout = w.QVBoxLayout(self._vscrollContent)
        self._vscrollLayout.setContentsMargins(2,2,2,2)
        self._vscrollLayout.setSpacing(2)

        self._vboxlayout.addWidget(self.__vscrollArea)
        self._vboxlayout.addWidget(self.__commencingBlock)
        self._vboxlayout.setAlignment(Qt.AlignBottom)
        self._vscrollLayout.setAlignment(Qt.AlignTop)
        self._vboxlayout.setContentsMargins(0,0,0,0)
        self._vboxlayout.setSpacing(3)

    def __initScrolContent(self):
        self.__vscrollArea = QScrollArea(self)
        self.__vscrollArea.setWidgetResizable(True)
        self._vscrollContent = QWidget(self.__vscrollArea)
        self._vscrollContent.setGeometry(QRect(0, 0, 600, 300))

        self.__vscrollArea.setWidget(self._vscrollContent)
        self.__vscrollArea.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)




    def __initCommencingBlock(self):

        self.__commencingBlock = w.QWidget()
        layout = w.QHBoxLayout(self.__commencingBlock)

        checkwidget = QWidget()
        checklayout = QHBoxLayout(checkwidget)
        checklayout.setAlignment(Qt.AlignCenter)
        self.__autoCheckBox = w.QCheckBox()
        self.__autoCheckBox.setChecked(False)
        self.__autoCheckBox.stateChanged.connect(self.__checkBoxChanged)
        checklayout.addWidget(self.__autoCheckBox)

        checklayout.addWidget(QLabel("Perform all task in the workflow"))
        layout.addWidget(checkwidget)

        self.__startButton = w.QPushButton()
        self.__startButton.setText("Start")
        self.__startButton.setFixedSize(100,50)
        self.__startButton.clicked.connect(self.__startPressed)
        layout.addWidget(self.__startButton)

        self.__saveProfileButton = w.QPushButton()
        self.__saveProfileButton.setText("Save Profile")
        self.__saveProfileButton.setFixedSize(100,50)
        layout.addWidget(self.__saveProfileButton)

        self.__addAlgoButton = w.QPushButton()
        self.__addAlgoButton.setText("Add Tools")
        self.__addAlgoButton.setFixedSize(100, 50)
        self.__addAlgoButton.clicked.connect(self._addTool)
        layout.addWidget(self.__addAlgoButton)

        layout.setAlignment(Qt.AlignCenter)
        self.__commencingBlock.setLayout(layout)

    def _addTool(self):
        print("Overload this one")


    def deleteTool(self, widget):
        self._toolQueue.remove(widget)
        self._vboxlayout.removeWidget(widget)


    def setAutoWorkflow(self, param):
        if(type(param) is not type(True)):
            return
        self.__autoCheckBox.setChecked(param)

    def __startPressed(self):
        if(DEBUGGING):
            print("StartPressed\n")


        if(self.__autoCheckBox.isChecked()):
            self.__mainpage.runWorkflow()
            self.__mainpage.updateProject()
            return

        for i in self._toolQueue:
            cmd = i.runCommand()
        self.__mainpage.updateProject()



    def setProjectManager(self, manager):
        self.__projectmanager = manager

    def requestProjectDirectory(self):
        return self.__projectmanager.getDirectory()

    def requestFileList(self):
        return self.__projectmanager.getFileList()

    def __checkBoxChanged(self):
        self.__mainpage.checkBoxChanged(self.__autoCheckBox.isChecked())

    def getToolQueue(self):
        return self._toolQueue

    def requestExpectedOutput(self, wf, id):
        return self.__mainpage.requestExpectedOutput(wf, id, self._toolQueue)

    def setWidgetEnable(self, state):
        if(type(state) != type(True)):
            return
        self.__startButton.setEnabled(True)

    def setInitialID(self, id):
        self._initID = id

    def getInitialID(self):
        return self._initID



from PyQt5 import QtWidgets as w


class FileViewer (w.QWidget):


    def __init__(self, file, p):
        super().__init__()

        self.__file = file
        self.__unsaved = False
        self.__initUI__()



    def __initUI__(self):

        # INIT LAYOUT
        self.__layout = w.QVBoxLayout(self)


        # INIT TEXT EDITOR
        self.__textEdit = w.QTextEdit(self)
        self.__textEdit.setStyleSheet('background-color:white; color:black;')
        self.__textEdit.setFrameStyle(0)
        self.__textEdit.setDisabled(True)

        # SET CONTENT IN THE FILE
        with open(self.__file) as f:
            self.__textEdit.setText(f.read())

        self.__textEdit.textChanged.connect(self.__textChanged)
        self.__textEdit.show()

        self.setStyleSheet("background-color: white;")

        self.__layout.setContentsMargins(0,0,0,0)
        self.__layout.addWidget(self.__textEdit)
        self.setLayout(self.__layout)

        self.show()

    def getText(self):
        return self.__textEdit.toPlainText()

    def __textChanged(self):
        self.__unsaved = True

    def isUnsaved(self):
        return self.__unsaved

    def getFile(self):
        return self.__file


import PyQt5.QtWidgets as w
from PyQt5.QtCore import *
import pymirna.UI.SettingBlock as s
from pymirna.UI.mComboBox import *

from pymirna.UI.TargetPredictionToolBlock import *
from pymirna.UI.WorkflowUI import *


class TargetPrediction(WorkflowUI):

    def __init__(self, p):
        super(TargetPrediction, self).__init__(p)


    def _addTool(self):
        pptb = TargetPredictionToolBlock(self, [self._initID, self._toolcount])
        pptb.setParent(self._vscrollContent)
        self._vscrollLayout.addWidget(pptb)
        pptb.show()
        print(self._vscrollLayout.count())
        self._toolQueue.append(pptb)
        self._initID += 1
        self._toolcount += 1


from PyQt5.QtWidgets import *
from pymirna.UI.mComboBox import *
from pymirna.UI.CutadaptUI import *
from pymirna.UI.ToolBlock import *

class DiscoveryToolBlock(ToolBlock):
    def __init__(self, ppui, count):
        super(DiscoveryToolBlock, self).__init__(ppui, count)
        self._workflowStep = DISCOVERY
        self._algorithmCombobox.addItem("mirDeep2")


    def ChangeWidget(self):
        print("Widget Count : "+str(self.__vboxlayout.count()))

        if(self.__algorithmCombobox.getData() == "None"):
            widget = self.__vboxlayout.takeAt(self.__vboxlayout.count()-1).widget()
            widget.setVisible(False)
            widget = None
            self.__vboxlayout.addWidget(QWidget())
            self.__vboxlayout.update()
            self.__curalgo = None
        elif(self.__algorithmCombobox.getData() == "mirDeep2"):
            widget = self.__vboxlayout.takeAt(self.__vboxlayout.count()-1).widget()
            widget.setVisible(False)

            #TODO CHANGE THIS
            ca = CutadaptUI(self)
            self.__vboxlayout.addWidget(ca)
            self.__curalgo = ca
            self.__vboxlayout.update()




"""miRNA Python Library

a collection of modules for dealing with microRNA data in Python.

"""

__version__ = "0.1.0"
__author__ = "kulwadee"

__all__=["UI","Utility"]

def info():
   return ("miRNA Python Libray, version %s" % __version__)

import os
import sys
import xml.etree.cElementTree
import subprocess


from pymirna.UI.MainPage import *
from pymirna.UI.PreProcessUI import *
from pymirna.UI.PreProcessUI import *
from pymirna.Utility import *
from pymirna.UI.SelectInputDialog import *

def runApp():
    app = w.QApplication([])
    app.setApplicationName("miRWB")
    mainpage = MainPage()
    mainpage.show()
    sys.exit(app.exec_())



from PyQt5.QtWidgets import *
from pymirna.UI.mComboBox import *
from pymirna.UI.CutadaptUI import *

PREPROCESSING = 1
DISCOVERY = 2
TARGETPREDICTION = 3

class ToolBlock(QFrame):
    def __init__(self, ppui, count):
        super(ToolBlock, self).__init__()
        self.__ppui = ppui
        self.__initAttribute(count)
        self.__initUI()

    def __initAttribute(self, count):
        self.__algorithmBlock = None
        self._algorithmCombobox = None
        self.__profileBlock = None
        self.__profileCombobox = None
        self.__saveProfileButton = None
        self.__deleteButtonWidget = None
        self.__deleteToolButton = None
        self._vboxlayout = None
        self.__ID = count[0]
        self.__count = count[1]
        self._curalgo = None
        self._workflowStep = PREPROCESSING

    def __initUI(self):
        self.__initAlgorithm()
        self.__initProfile()

        self.__deleteButtonWidget = QWidget()
        self.__deleteButtonWidget.setMinimumHeight(70)
        layout = QHBoxLayout(self.__deleteButtonWidget)
        self.__deleteToolButton = QPushButton()
        self.__deleteToolButton.setText("Remove")
        self.__deleteToolButton.setFixedSize(100, 50)
        self.__deleteToolButton.clicked.connect(self.__deleteSelf)

        self.__saveProfileButton = QPushButton()
        self.__saveProfileButton.setText("Save Profile")
        self.__saveProfileButton.setFixedSize(100, 50)
        self.__saveProfileButton.clicked.connect(self.__saveProfile)

        layout.addWidget(QLabel("ID "+str(self.__ID)))
        layout.addWidget(self.__deleteToolButton)
        layout.addWidget(self.__saveProfileButton)
        self.__deleteButtonWidget.setLayout(layout)

        self.setFrameStyle(1)
        self.__initLayout()

    def __initAlgorithm(self):
        self.__algorithmBlock = QWidget()
        layout = QHBoxLayout(self.__algorithmBlock)
        self._algorithmCombobox = mComboBox()
        self._algorithmCombobox.addItem("None")
        self._algorithmCombobox.currentIndexChanged.connect(self.ChangeWidget)

        layout.addWidget(QLabel("Program"))
        layout.addWidget(self._algorithmCombobox)
        self.__algorithmBlock.setLayout(layout)

    def __initProfile(self):
        self.__profileBlock = QWidget()
        layout = QHBoxLayout(self.__profileBlock)
        self.__profileCombobox = mComboBox()
        self.__profileCombobox.addItem("None")

        layout.addWidget(QLabel("Profile"))
        layout.addWidget(self.__profileCombobox)
        self.__profileBlock.setLayout(layout)

    def __initLayout(self):
        self._vboxlayout = QVBoxLayout(self)

        self._vboxlayout.addWidget(self.__algorithmBlock)
        self._vboxlayout.addWidget(self.__profileBlock)
        self._vboxlayout.addWidget(self.__deleteButtonWidget)

        self._vboxlayout.addWidget(QWidget())

        self.setLayout(self._vboxlayout)

    def __deleteSelf(self):
        print("__deleteSelf")
        self.__ppui.deleteTool(self)
        self.setVisible(False)
        self = None

    def ChangeWidget(self):
        print("Overload This method")

    def __saveProfile(self):
        print("__saveProfile")
        print(self.getCommand())

    def runCommand(self):
        if(self._curalgo is not None):
            return self._curalgo.runCommand()
        return 0

    def getID(self):
        return self.__ID

    def getRank(self):
        return self.__count

    def requestProjectDirectory(self):
        return self.__ppui.requestProjectDirectory()

    def requestExpectedOutput(self):
        return self.__ppui.requestExpectedOutput( self._workflowStep, self.__ID)

    def requestFileList(self):
        return self.__ppui.requestFileList()

    def getExpectedOutput(self):
        return self._curalgo.getExpectedOutput()

    def getWorkflow(self):
        return self._workflowStep



# from distutils.core import setup
# from distutils.cmd import Command
#
# class TestCommand(Command):
#     user_options = []
#
#     def initialize_options(self):
#         pass
#     def finalize_options(self):
#         pass
#     def run(self):
#         pass
#
# setup(name='pymirna',
#       version='0.1',
#       description="pymirna: The Pythonic microRNA Library",
#       packages=['pymirna','pymirna.SeqIO'],
#       cmdclass={
#           'test': TestCommand
#       }
# )


#from distutils.core import setup
from setuptools import setup
from distutils.cmd import Command

class TestCommand(Command):
    user_options = []

    def initialize_options(self):
        pass
    def finalize_options(self):
        pass
    def run(self):
        pass

setup(name='pymirna',
      version='0.1.0',
      description="pymirna: The Pythonic microRNA Library",
      author='K. Somboonviwat',
      author_email='kulwadee.a@gmail.com',
      license='MIT',
      packages=['pymirna','pymirna.UI','pymirna.Utility'],
      install_requires=[
      'SIP', 'PyQt5',
      ],
      cmdclass={
          'test': TestCommand
      }
)



import os as o
import sys as s
import subprocess as sub


def isFile(file):
    if(o.path.isfile(file)):
        return True
    return False

def isFolder(folder):
    if(o.path.isdir(folder)):
        return True
    return False

def checkFile(file):
    if( isFolder(file) ):
        return False
    elif(not isFile(file) ):
        return False
    return True

def getFileExtension(file):
    temp = ""
    i = len(file)-1
    while(file[i] is not '.'):
        if(file[i] is '/'):
            return ''
        temp += file[i]
        i -= 1
    return temp[::-1]

def getFileName(file):
    temp = ""
    i = len(file)-1
    while(file[i] is not '/'):
        temp += file[i]
        i-=1

    return temp[::-1]

def getFilenameWithoutExtension(file):
    return getFileName(file).split("."+getFileExtension(file))[0]

def getDirectoryList(file):

    fn = getFileName(file)
    lis = file.replace(fn, '').split('/')
    lis.remove('')
    lis.remove('')
    return lis

def getFileList(curDir):
    lis = list()

    if(not o.path.isdir(curDir)):
        return list()

    if(curDir[len(curDir)-1] != '/'):
        curDir += '/'

    fileList = str(sub.check_output(["ls", curDir]))
    if(len(fileList) <= 0):
        return list()
    if('3.5' in s.version):
        fileList = fileList[2:len(fileList)-1]
        fileList = fileList.split('\\n')
        fileList.remove('')
    elif('2.7' in s.version):
        fileList = fileList.split('\n')
        fileList.remove('')

    for i in fileList:

        if(o.path.isfile(curDir + i)):
            lis.append(curDir + i)
        elif(o.path.isdir(curDir+i)):
            recurLis = getFileList(curDir+i+'/')
            for j in recurLis:
                lis.append(j)
    return lis


from pymirna.Utility.FileReader import *
import os


PROJECT_NAME_TAG = "[PROJECT_NAME]"
FILE_LIST_TAG = "[FILE_LIST]"
INITIAL_ID = "[INITIAL_ID]"
PROJECT_EXTENSION = ".pmi"

class ProjectManager:

    def __init__(self):
        self.__initAttribute__()


    def __initAttribute__(self):
        self.__filelist = []
        self.__projectName = ""
        self.__directory = ""
        self.__initID = [1,1,1]

    def createProject(self, path, name):
        if(path[len(path)-1] is not '/'):
            path += '/'

        self.__projectName = name
        self.__directory = path

        cmd = "mkdir -p " + path
        os.system(cmd)

        file = open(path+name+PROJECT_EXTENSION, 'w')
        file.writelines(PROJECT_NAME_TAG+'\n')
        file.writelines(name+'\n')
        file.writelines(FILE_LIST_TAG+'\n')
        file.writelines(INITIAL_ID+'\n')
        file.writelines('1,1,1'+'\n')
        file.close()

        cmd = "mkdir -p " + path + "Profile"
        os.system(cmd)
        cmd = "mkdir -p " + path + "Result/PreProcessing"
        os.system(cmd)
        cmd = "mkdir -p " + path + "Result/miRNA_Discovery"
        os.system(cmd)
        cmd = "mkdir -p " + path + "Result/Target_Prediction"
        os.system(cmd)
        cmd = "mkdir -p " + path + "file_list"
        os.system(cmd)

    def openProject(self, path):
        if( not checkFile(path) ):
            return False
        fext = '.'+getFileExtension(path)
        if( fext != PROJECT_EXTENSION ):
            return False

        readingFile = False

        file = open(path, 'r')
        temp = file.readline().split('\n')[0]
        while(temp is not ''):
            if(temp == PROJECT_NAME_TAG):
                self.__projectName = file.readline().split('\n')[0]
            elif(temp == FILE_LIST_TAG):
                readingFile = True
                temp = file.readline().split('\n')[0]
                continue
            elif(temp == INITIAL_ID):
                readingFile = False
                temp = file.readline().split('\n')[0].split(',')
                for i in range(0,len(temp)):
                    temp[i] = eval(temp[i])
                self.__initID = temp


            if(readingFile):
                self.__filelist.append(temp)

            temp = file.readline().split('\n')[0]

        self.__directory = path[0:len(path)-len(self.__projectName+PROJECT_EXTENSION):]
        file.close()

    def closeProject(self):
        self.__filelist = []
        self.__directory = ''
        self.__projectName = ''

    def saveProject(self):

        file = open(self.__directory+self.__projectName+PROJECT_EXTENSION, 'w')

        file.writelines(PROJECT_NAME_TAG+'\n')
        file.writelines(self.__projectName+'\n')
        file.writelines(FILE_LIST_TAG+'\n')
        for i in self.__filelist:
            file.writelines(i+'\n')

        file.writelines(INITIAL_ID+'\n')
        file.writelines(str(self.__initID[0])+','+str(self.__initID[1])+','+str(self.__initID[2])+'\n')
        file.close()

    def getFileList(self):
        return self.__filelist

    def addFile(self, file):
        if(not checkFile(file)):
            return False
        self.__filelist.append(file)

    def removeFile(self, file):
        for i in self.__filelist:
            if(i == file):
                self.__filelist.remove(i)
                return

    def getProjectName(self):
        return self.__projectName

    def changeProjectName(self, n):
        self.__projectName = n

    def getDirectory(self):
        return self.__directory

    def setInitialID(self, idlist):
        self.__initID = idlist

    def getInitialID(self):
        return self.__initID